"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const utils_1 = require("@umijs/utils");
// @ts-ignore
const ncc_1 = __importDefault(require("@vercel/ncc"));
const config_1 = require("./config");
const api_extractor_1 = require("@microsoft/api-extractor");
exports.default = async (opts) => {
    // patch @microsoft/api-extractor before prepare config
    // use require() rather than import(), to avoid jest runner to fail
    // ref: https://github.com/nodejs/node/issues/35889
    require('./patcher');
    const config = (0, config_1.getConfig)(opts);
    const count = Object.keys(config.deps).length;
    const startTime = Date.now();
    // bundle deps
    for (const dep in config.deps) {
        const { output, pkg, nccConfig } = config.deps[dep];
        const outputDir = path_1.default.dirname(output);
        utils_1.logger.info(`Pre-bundle ${utils_1.chalk.yellow(pkg.name)} to ${utils_1.chalk.yellow((0, utils_1.winPath)(path_1.default.relative(opts.cwd, outputDir)))}`);
        await (0, ncc_1.default)(dep, nccConfig).then(({ code }) => {
            // create dist path
            if (!fs_1.default.existsSync(outputDir)) {
                fs_1.default.mkdirSync(outputDir, { recursive: true });
            }
            // TODO: dist content validate
            // emit dist file
            fs_1.default.writeFileSync(output, code, 'utf-8');
            // emit package.json
            fs_1.default.writeFileSync(path_1.default.join(output, '../package.json'), JSON.stringify({
                name: pkg.name,
                author: pkg.author,
                license: pkg.license,
            }), 'utf-8');
        });
    }
    // bundle dts
    Object.keys(config.dts).length &&
        utils_1.logger.event(`Generate declaration files...`);
    for (const dts in config.dts) {
        api_extractor_1.Extractor.invoke(config.dts[dts].maeConfig, {
            localBuild: true,
            showVerboseMessages: false,
        });
    }
    if (count) {
        utils_1.logger.event(`Pre-bundled successfully in ${Date.now() - startTime} ms (${count} deps)`);
    }
};
